/*

  Program Predator-Prey.ccp:
  solves the equations of the Lotka-Volterra system.
  Equations:

  d p / dt = a * p - c * p * P
  d P / dt = - b * P + e * p * P

 */

#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

#include "RK2.h"
#include "../Integrals/Integrals.h"


// Global variables (parameters of the equation, to use in dydt!)

double a, c, b, e, eps;


// Forward defined function dydt and calc_max

void dydt( int, double, double *, double * );
void calc_max( double, double *, long, long&, long& );


/*

  Main program

*/

int main()
{

  cout << "Program to integrate the Lotka-Volterra system" << endl;
  cout << "through a second order Runge-Kutta time scheme." << endl;

  int Neq = 2;

  cout << "Value of dt, Tend, Tprint: ";
  double dt, Tend, Tprint;
  cin >> dt >> Tend >> Tprint;

  long Nend   = long( Tend / dt );
  long Nprint = long( Tprint / dt );

  double epsf;
  cout << "Parameters a, c, b, e and eps: ";
  cin >> a >> c >> b >> e >> epsf;

  cout << "Initial conditions p0, P0: ";
  double y[Neq];
  cin >> y[0] >> y[1];

  RK2 sol = RK2( Neq, y );

  cout << "Output file name: ";
  char nfile[256];
  cin >> nfile;

  double t_n = 0.0;
  eps = 0.0;


  // Open the output file and writes the initial conditions

  ofstream fout;
  fout.open( nfile );
  fout.precision( 15 );
  fout << t_n;
  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
  fout << endl;

  // Time loop

  for ( long n = 1; n <= Nend; n++ )
    {

      if ( t_n >= Tend*0.5 ) eps = epsf;

      // One step forward through the RK2 scheme

      sol.stepRK2( t_n, dt, dydt );
      t_n = n * dt;

      // Print each Tprint time intervals

      if ( ( n % Nprint ) == 0 )
	{
	  sol.getValue( y );
	  fout << t_n;
	  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
	  fout << endl;
	}

    }

  fout.close();


  // Reads data just written on the file to compute the periods

  long Nout = long( Tend / Tprint ) + 1;
  double *tt = new double[ Nout ];
  double *pp = new double[ Nout ];
  double *PP = new double[ Nout ];
  ifstream finp;
  finp.open( nfile );
  for ( long i = 0; i < Nout; i++ )
    {
      finp >> tt[i] >> pp[i] >> PP[i];
    }
  finp.close();

  // Computes the positions of maxima to compute the period

  long p1, p2, P1, P2;

  calc_max( Tprint, pp, Nout, p1, p2 );
  cout << "Position of maxima for the preys: " << p1 << ", " << p2 << endl;
  double Tper = tt[p2] - tt[p1];
  cout << "Period for the preys: " << Tper << endl;
  long Npp = p2 - p1 + 1;
  double *fint = new double[Npp];
  for ( long i = p1; i <= p2; i++ ) fint[i-p1] = pp[i];
  cout << "<p> / T: " << Trapezoidal( fint, Tprint, Npp ) / Tper << endl;
  cout << "b / e: " << b / e << endl << endl;
  delete [] fint;

  calc_max( Tprint, PP, Nout, P1, P2 );
  cout << "Positions of maxima for predators: " << P1 << ", " << P2 << endl;
  Tper = tt[P2] - tt[P1];
  cout << "Period for predators: " << Tper << endl;
  Npp = P2 - P1 + 1;
  fint = new double[Npp];
  for ( long i = P1; i <= P2; i++ ) fint[i-P1] = PP[i];
  cout << "<P> / T: " << Trapezoidal( fint, Tprint, Npp-1 ) / Tper << endl;
  cout << "a / c: " << a / c << endl;
  delete [] fint;

}



// Function dydt: computes the RHS of the system of equations

void dydt( int Neq, double t, double * y, double * dy )
{
  dy[0] = a * y[0] - c * y[0] * y[1] - eps * y[0];
  dy[1] = - b * y[1] + e * y[0] * y[1] - eps * y[1];
}


// Function calc_max: computes the position of the two consecutive maxima in a vector

void calc_max( double dt, double *v, long N, long& m1, long& m2 )
{
  double der1, der2, der1p;

  der1p = ( v[2] - v[0] ) / ( 2.0 * dt );
  long i = 2;
  int max_found = 0;
  while ( ( i < N - 1 ) && ( max_found < 2 ) )
    {
      der1 = ( v[i+1] - v[i-1] ) / ( 2.0 * dt );
      der2 = ( v[i+1] + v[i-1] - 2.0 * v[i] ) / ( dt * dt );
      if ( ( der1 * der1p <= 0.0 ) && ( der2 < 0.0 ) )
	{
	  max_found++;
	  if ( max_found == 1 ) m1 = i;
	  if ( max_found == 2 ) m2 = i;
	}
      der1p = der1;
      i++;
    }
}
