/*

  Program Planets.ccp:
  integrate the motion equation for a planet orbiting around the Sun.
  Equations:

  d^2 x / dt^2 = - G * x / R^3
  d^2 y / dt^2 = - G * y / R^3

 */

#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

#include "RK2.h"


// Global variables (parameters of the equation, to use in dydt!)

double G;


// Forward defined function dydt

void dydt( int, double, double *, double * );


/*

  Main program

*/

int main()
{

  cout << "Program to integrate the motion equation of a planet orbiting around the Sun" << endl;
  cout << "through a second order Runge-Kutta time scheme." << endl;

  int Neq = 4;

  cout << "Value of dt, Tend, Tprint: ";
  double dt, Tend, Tprint;
  cin >> dt >> Tend >> Tprint;

  long Nend   = long( Tend / dt );
  long Nprint = long( Tprint / dt );

  cout << "Parameter G in the equations: ";
  cin >> G;

  cout << "Initial conditions x0, vx0, y0, vy0: ";
  double y[Neq];
  cin >> y[0] >> y[1] >> y[2] >> y[3];

  RK2 sol = RK2( Neq, y );

  cout << "Output file name: ";
  char nfile[256];
  cin >> nfile;

  double t_n = 0.0;


  // Physical invariants: Energy and Angular momentum

  double R  = sqrt( y[0] * y[0] + y[2] * y[2] );
  double E  = 0.5 * ( y[1] * y[1] + y[3] * y[3] ) - G / R;
  double Lz = y[0] * y[3] - y[2] * y[1];


  // Open the output file and writes the initial conditions

  ofstream fout;
  fout.open( nfile );
  fout.precision( 15 );
  fout << "#   t     x    vx    y    vy   E   Lz" << endl;
  fout << t_n;
  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
  fout << "   " << E << "   " << Lz;
  fout << endl;

  // Time loop

  for ( long n = 1; n <= Nend; n++ )
    {

      // One step forward through the RK2 scheme

      sol.stepRK2( t_n, dt, dydt );
      t_n = n * dt;

      // Print each Tprint time intervals

      if ( ( n % Nprint ) == 0 )
	{
	  sol.getValue( y );
	  R  = sqrt( y[0] * y[0] + y[2] * y[2] );
	  E  = 0.5 * ( y[1] * y[1] + y[3] * y[3] ) - G / R;
	  Lz = y[0] * y[3] - y[2] * y[1];
	  fout << t_n;
	  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
	  fout << "   " << E << "   " << Lz;
	  fout << endl;
	}

    }

  fout.close();

}


// Function dydt: computes the RHS of the system of equations

void dydt( int Neq, double t, double * y, double * dy )
{
  double R = sqrt( y[0] * y[0] + y[2] * y[2] );
  double R3 = R * R * R;

  dy[0] = y[1];
  dy[1] = - G * y[0] / R3;
  dy[2] = y[3];
  dy[3] = - G * y[2] / R3;
}
